/**
 * Copyright (c) 2011, REMPL.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1) Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2) Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following
 * disclaimer in the documentation and/or other materials provided
 * with the distribution.
 * 3) Neither the name of the REMPL.com nor the names of its
 * contributors may be used to endorse or promote products derived
 * from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.rempl.readers.its.dbl;

// BRIDGE API
import com.rempl.readers.its.bridge.BridgeException;
import com.rempl.readers.its.bridge.Change;
import com.rempl.readers.its.bridge.ChangeAssignee;
import com.rempl.readers.its.bridge.ChangeMessage;
import com.rempl.readers.its.bridge.ChangeStatus;
import com.rempl.readers.its.bridge.ChangeTitle;
import com.rempl.readers.its.bridge.Issue;
import com.rempl.readers.its.bridge.Person;

// autogenerated JAXB classes
import com.rempl.readers.its.dbl.jaxb.XmlChangeAssignee;
import com.rempl.readers.its.dbl.jaxb.XmlChangeMessage;
import com.rempl.readers.its.dbl.jaxb.XmlChangeStatus;
import com.rempl.readers.its.dbl.jaxb.XmlChangeTitle;
import com.rempl.readers.its.dbl.jaxb.XmlIssue;
import com.rempl.readers.its.dbl.jaxb.XmlStatus;

// utils
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Mocked/stubbed version of an issue.
 *
 * @author Yegor Bugayenko (yegor@rempl.com)
 * @version $Id$
 */
public final class IssueDbl implements Issue {

    /**
     * List of changes.
     */
    private Collection<Change> changes;

    /**
     * Protected ctor, available inside the package only.
     * @param issue The issue just discovered in XML
     * @see BridgeDbl#issues()
     * @throws BridgeException If something goes wrong
     */
    protected IssueDbl(final XmlIssue issue) throws BridgeException {
        final List<Object> xmlChanges =
            issue.getChangeAssigneeOrChangeMessageOrChangeStatus();
        this.changes = new ArrayList<Change>();
        for (Object xmlChange : xmlChanges) {
            this.changes.add(this.change(xmlChange));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Collection<Change> changes() throws BridgeException {
        return this.changes;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Change latest(final Class type) throws BridgeException {
        Change latest = null;
        for (Change change : this.changes()) {
            final boolean matches = change.getClass().equals(type)
                && (latest == null || latest.time().before(change.time()));
            if (matches) {
                latest = change;
            }
        }
        assert latest != null;
        return latest;
    }

    /**
     * Convert XML change into Bridge API change.
     * @param obj The change retrieved from XML by JAXB
     * @return The Bridge API change
     * @see #IssueDbl(XmlIssue)
     * @throws BridgeException If something goes wrong
     */
    private Change change(final Object obj)
        throws BridgeException {
        Change change = null;
        if (obj instanceof XmlChangeAssignee) {
            final XmlChangeAssignee xmlChange = (XmlChangeAssignee) obj;
            change = new ChangeAssignee(
                new Person(xmlChange.getAuthor()),
                xmlChange.getTime().getTime(),
                new Person(xmlChange.getAssignee())
            );
        } else if (obj instanceof XmlChangeMessage) {
            final XmlChangeMessage xmlChange = (XmlChangeMessage) obj;
            change = new ChangeMessage(
                new Person(xmlChange.getAuthor()),
                xmlChange.getTime().getTime(),
                xmlChange.getMessage()
            );
        } else if (obj instanceof XmlChangeStatus) {
            final XmlChangeStatus xmlChange = (XmlChangeStatus) obj;
            change = new ChangeStatus(
                new Person(xmlChange.getAuthor()),
                xmlChange.getTime().getTime(),
                this.status(xmlChange.getStatus())
            );
        } else if (obj instanceof XmlChangeTitle) {
            final XmlChangeTitle xmlChange = (XmlChangeTitle) obj;
            change = new ChangeTitle(
                new Person(xmlChange.getAuthor()),
                xmlChange.getTime().getTime(),
                xmlChange.getTitle()
            );
        } else {
            throw new BridgeException("Invalid type of change");
        }
        return change;
    }

    /**
     * Convert XML status into Bridge API status.
     * @param sts The status from XML JAXB
     * @return The Bridge API status
     * @see #change(Object)
     * @throws BridgeException If something goes wrong
     */
    private ChangeStatus.Status status(final XmlStatus sts)
        throws BridgeException {
        ChangeStatus.Status status;
        if (sts == XmlStatus.OPEN) {
            status = ChangeStatus.Status.OPEN;
        } else if (sts == XmlStatus.FIXED) {
            status = ChangeStatus.Status.FIXED;
        } else if (sts == XmlStatus.INVALID) {
            status = ChangeStatus.Status.INVALID;
        } else {
            status = ChangeStatus.Status.DUPLICATE;
        }
        return status;
    }

}
